from django.contrib.auth.models import User, check_password
from django.conf import settings
from janrain.django.models import CaptureUser
from janrain.django.capture import CaptureApi, CaptureApiError, get_attribute_map

import logging
logger = logging.getLogger(__name__)

class CaptureBackend(object):
    """
    Janrain Capture Authentication Backend
    """
    def authenticate(self, access_token):
        """
        Authenticate

        Authenticate users with an access_token provisioned by the Capture widget.
        If the user does not yet exist in Django then they will be created but will
        not be activated.

        Args:
            access_token - Access token provisioned by the Janrain Capture widget.
        """
        attribute_map = get_attribute_map()
        attributes = ['uuid',] + attribute_map.values()

        try:
            api = CaptureApi()
            entity = api.entity(access_token, attributes)
        except CaptureApiError as error:
            logger.warn(error.message)
            return None

        try:
            user = CaptureUser.objects.get(uuid=entity['uuid']).user

            # Sync user attributes from Capture record at login
            for user_attr, entity_attr in attribute_map.items():
                setattr(user, user_attr, entity[entity_attr])
            user.save()

            return user

        except CaptureUser.DoesNotExist:
            # Create user
            user = User()

            for user_attr, entity_attr in attribute_map.items():
                setattr(user, user_attr, entity[entity_attr])

            user.set_unusable_password()

            user.is_active = getattr(settings, 'JANRAIN_ACTIVATE_NEW_USERS', False)
            user.is_staff = False
            user.is_superuser = False
            user.save()

            # Create CaptureUser to link the new User to the Capture UUID
            capture_user = CaptureUser(user=user, uuid=entity['uuid'])
            capture_user.save()

            return user

        return None

    def get_user(self, user_id):
        """ Get a User object based on the user_id. """
        try:
            return User.objects.get(pk=user_id)
        except User.DoesNotExist:
            return None



from django.conf import settings
from django.http import HttpResponseRedirect, HttpResponse
from django.core.urlresolvers import reverse

def login_required(view_func):
    """
    Redirect users to 'janrain-sign-in' if they are not authenticated.
    """
    def _janrain_user_required(request, *args, **kwargs):
        if not request.user.is_authenticated():
            # TODO: Configurable URL/named URL pattern
            uri = reverse("janrain-sign-in")
            if '?' in uri:
                uri += '&'
            else:
                uri += '?'
            uri += "next=" + request.get_full_path()
            return HttpResponseRedirect(uri)
        return view_func(request, *args, **kwargs)
    return _janrain_user_required



from django.conf.urls import patterns, include, url
from django.views.generic import TemplateView
from django.core.urlresolvers import reverse

urlpatterns = patterns('janrain.django.views',
    url(r'^sign-in/$', 'sign_in', name='janrain-sign-in'),
    url(r'^sign-out/$', 'sign_out', name='janrain-sign-out'),
)



from django.conf import settings
from django.db import models

"""
Capture User

Stores the relationship between a Django user and a Capture record.
"""
class CaptureUser(models.Model):
    user = models.ForeignKey(settings.AUTH_USER_MODEL)
    uuid = models.CharField(max_length=38, db_index=True, unique=True,
                            editable=False)

import signals



from django.conf import settings
from janrain.capture import Api, ApiResponseError

import logging
logger = logging.getLogger(__name__)

# Make sure the required JANRAIN_CAPTURE setting is defined
if not hasattr(settings, 'JANRAIN_CAPTURE'):
    raise ImproperlyConfigured("You need to specify JANRAIN_CAPTURE in " \
                               "your Django settings file.")

class CaptureApiError(ApiResponseError):
    pass

class CaptureApi(Api):
    def __init__(self):
        super(CaptureApi, self).__init__(settings.JANRAIN_CAPTURE['server'])
        self.logger = logger

    def call(self, api_call, **kwargs):
        # wrap the call method so we can raise a local exception instead
        try:
            return super(CaptureApi, self).call(api_call, **kwargs)
        except ApiResponseError as error:
            raise CaptureApiError(error.code, error.error, str(error), error.response)

    def entity(self, access_token, attributes=None):
        """
        Get the entity scoped to the specified access_token.

        Args:
            access_token - A Capture access token
            attributes   - A list of attributes to include in response

        Returns:
            The entity as a dictionary
        """
        kwargs = {
            'type_name': get_entity_type(),
            'access_token': access_token
        }

        if attributes:
            kwargs['attributes'] = attributes

        response = self.call("/entity", **kwargs)

        return response['result']

    def oauth_token(self, code, redirect_uri):
        self.sign_requests = False
        response = self.call("/oauth/token", code=code,
                            client_id=settings.JANRAIN_CAPTURE['client_id'],
                            client_secret=settings.JANRAIN_CAPTURE['client_secret'],
                            grant_type='authorization_code',
                            redirect_uri=redirect_uri)
        logger.debug(response)

        return response

    def oauth_refresh_token(self, refresh_token):
        """
        Call /oauth/token to refresh the current token.

        Args:
            refresh_token - The current refresh token, typically saved in the session.

        Returns:
            The API response as a dictionary
        """
        self.sign_requests = False
        response = self.call("/oauth/token",
                             refresh_token=refresh_token,
                             client_id=settings.JANRAIN_CAPTURE['client_id'],
                             client_secret=settings.JANRAIN_CAPTURE['client_secret'],
                             grant_type='refresh_token')
        logger.debug(response)

        return response

def get_attribute_map():
    """
    Return the mapping from Django's User object fields to Capture attributes.
    """
    default_map = {
        'username': 'displayName',
        'email': 'email',
        'first_name': 'givenName',
        'last_name': 'familyName'
    }

    return getattr(settings, 'JANRAIN_CAPTURE_ATTRIBUTE_MAP', default_map)

def get_entity_type():
    """
    Return the entity type (aka. type name) for Capture.
    """

    return getattr(settings, 'JANRAIN_CAPTURE_ENTITY_TYPE', "user")




from django.http import HttpResponseNotAllowed
from django.template import RequestContext
from django.template import loader
from django.conf import settings
from django.contrib.auth import logout
from datetime import datetime
from janrain.django.session import refresh_session

import logging
logger = logging.getLogger(__name__)

class RefreshTokenMiddleware(object):
    def process_request(self, request):
        if request.user.is_authenticated() and self.token_is_expired(request):
            try:
                logger.debug("Refreshing Capture access token")
                refresh_session(request.session)
            except Exception as error:
                logger.warn(str(error))
                logout(request)

        return None

    def token_is_expired(self, request):
        if 'janrain_expires' in request.session:
            return datetime.now() >= request.session['janrain_expires']




from django.conf import settings
from django.contrib.auth.models import User
from django.dispatch import receiver
from django.db.models.signals import post_save
from janrain.capture import Api, ApiResponseError
from janrain.django.capture import get_attribute_map, get_entity_type
from janrain.django.models import CaptureUser

"""
Update Capture User

When a User record is updated the corresponding fields are also updated in the
Capture record.
"""
@receiver(post_save, sender=User)
def update_capture_user(sender, **kwargs):
    # Not necessary to update Capture on create since Capture provides the data
    # initially in the authentication backend.
    if not kwargs['created']:
        user = kwargs['instance']
        try:
            capture_user = CaptureUser.objects.get(user=user)
        except CaptureUser.DoesNotExist:
            return

        attribute_map = get_attribute_map()
        attributes = {}

        for user_attr, entity_attr in attribute_map.items():
            attributes[entity_attr] = getattr(user, user_attr)

        # Use client_id and client_secret since this update may be initiated by
        # a Django administrator.
        api = Api(settings.JANRAIN_CAPTURE['server'])
        response = api.call("/entity.update", type_name=get_entity_type(),
                            uuid=capture_user.uuid,
                            client_id=settings.JANRAIN_CAPTURE['client_id'],
                            client_secret=settings.JANRAIN_CAPTURE['client_secret'],
                            value=attributes)




#!/usr/bin/env python
import distribute_setup
distribute_setup.use_setuptools()

import os
from setuptools import setup, find_packages

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

from janrain.django import get_version

setup(
    name = "janrain-django",
    version = get_version(),
    description = "Janrain integration with Django",
    long_description = read("README.rst"),
    author = "Micah Carrick",
    author_email = "micah@janrain.com",
    url = "http://developers.janrain.com/",
    packages = find_packages(),
    namespace_packages = ["janrain"],
    classifiers = [
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Internet :: WWW/HTTP",
        "License :: OSI Approved :: MIT License"
    ],
    #test_suite = "janrain.django.test"
    install_requires = [
        'janrain-python-api >= 0.2.1',
    ],
)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from datetime import datetime, timedelta
from janrain.django.capture import CaptureApi, CaptureApiError

def set_session(session, api_response):
    # Save new tokens in the session
    session['janrain_access_token'] = api_response['access_token']
    session['janrain_refresh_token'] = api_response['refresh_token']
    expires = datetime.now() + timedelta(0, api_response['expires_in'])
    session['janrain_expires'] = expires

def refresh_session(session):
    # Refresh the token
    api = CaptureApi()
    token = session['janrain_refresh_token']
    api_response = api.oauth_refresh_token(token)

    set_session(session, api_response)



from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login, logout
from django.shortcuts import redirect, render
from django.core.exceptions import ImproperlyConfigured
from janrain.django.capture import CaptureApi, CaptureApiError
from janrain.django.session import set_session

import logging
logger = logging.getLogger(__name__)

def sign_in(request):
    # The same redirect URI needs to be set in the Javascript settings for
    # Capture as well as passed to the oauth/token API call.
    query_start = request.build_absolute_uri().find('?')
    redirect_uri = request.build_absolute_uri()[0:query_start].strip('/')

    # Store the next URL in the session so that we don't have to pass it around
    # through the Capture redirects.
    if 'next' in request.GET:
        request.session['next'] = request.GET['next']

    context = {'redirect_uri': redirect_uri}

    if 'code' in request.GET:
        # Exchange the code for access and refresh tokens.
        api = CaptureApi()
        try:
            api_response = api.oauth_token(request.GET['code'], redirect_uri)
        except CaptureApiError as error:
            logger.warn(str(error))
            messages.error(request, "Error: " + str(error))
            return redirect('janrain-sign-in')

        user = authenticate(access_token=api_response['access_token'])

        if not user:
            # Yes, it's not a very helpful error message but it's an edge case
            messages.error(request, "Authentication failed.")
            return redirect('janrain-sign-in')

        if user.is_active:
            login(request, user)
            set_session(request.session, api_response)

            if 'next' in request.session:
                next = request.session['next']
                del request.session['next']
                return redirect(next)
            elif hasattr(settings, 'JANRAIN_SIGN_IN_REDIRECT'):
                return redirect(settings.JANRAIN_SIGN_IN_REDIRECT)
        else:
            messages.error(request, "That account has not been activated.")
            return redirect('janrain-sign-in')

    return render(request, "janrain/sign-in.html", context)

def sign_out(request):
    logout(request)

    if hasattr(settings, 'JANRAIN_SIGN_OUT_REDIRECT'):
        return redirect(settings.JANRAIN_SIGN_OUT_REDIRECT)

    return render(request, "janrain/sign-out.html", {})


